
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

.include "cpctelera.h.s"
.include "collision.h.s"
.include "functions.h.s"
.include "bullet.h.s"
.include "assets.h.s"
.include "macros.h.s"

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PRIVATE DATA
;; =====================================================================================================================||
;; =====================================================================================================================||

;; Barril Definition
crearBarril barril1, #60,  #50, #8, #8, #_pre_roca, #10
crearBarril barril2, #60, #100, #8, #8, #_pre_roca, #10

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PUBLIC FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Inicializamos los barriles
;; ENTRADA: --
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
barril_init_all::

	ld		ix,	#barril1_variables		;;| Cargamos el prime barril y lo inicializamos
	call	barril_init					;;| ( IX / AF, BC, DE, HL, IY / - )

	ld		ix,	#barril2_variables		;;| Cargamos el segundo barril y lo inicializamos
	call	barril_init					;;| ( IX / AF, BC, DE, HL, IY / - )

	ld		barril_y(ix),		#100	;;  Cambiamos la posicion Y del segudo barril para que no vayan a la par

	ret
;; ======================================================================
;; *Update de todos los barriles para moverlos
;; ENTRADA: --
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
barril_update::
	ld		ix, #barril1_variables		;;| Cargamos el primer barril y lo movemos
	call	move_barril					;;| ( IX / AF, BC, DE, HL, IY / - )

	ld		ix, #barril2_variables		;;| Cargamos el segundo barril y lo movemos
	call	move_barril					;;| ( IX / AF, BC, DE, HL, IY / - )

	ret
;; ======================================================================
;; *Guarda la posicion de los barriles del frame anterior
;; ENTRADA: --
;; MODIFICA: IX, A
;; SALIDA: --
;; ======================================================================
save_last_variables_barril::

	ld		ix, #barril1_variables		;;| Cargamos el primer barril y guardamos su posicion
	call	save_last_variables			;;| ( IX / A / - )

	ld		ix, #barril2_variables		;;| Cargamos el segundo barril y guardamos su posicion
	call	save_last_variables			;;| ( IX / A / - )

	ret
;; ======================================================================
;; *Borramos los barriles
;; ENTRADA: -- 
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
barril_erase::
	ld 		ix, #barril1_variables		;;| Cargamos el primer barril y lo borramos
	call 	erase_entity				;;| ( IX / AF, BC, DE, HL / - )

	ld 		ix, #barril2_variables		;;| Cargamos el segundo barril y lo borramos
	call 	erase_entity				;;| ( IX / AF, BC, DE, HL / - )

	ret
;; ======================================================================
;; *Pintamos los barriles
;; ENTRADA: --
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
barril_draw::
	ld 		ix, #barril1_variables		;;| Cargamos el primer barril y lo pintamos
	call 	paint_entity				;;| ( IX / AF, BC, DE, HL / - )

	ld 		ix, #barril2_variables		;;| Cargamos el segundo barril y lo pintamos
	call 	paint_entity				;;| ( IX / AF, BC, DE, HL / - )

	ret

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PRIVATE FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Inicializa las variables del barril
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
barril_init:
	ld		barril_y(ix), #0				;; Movemos el barril a la parte superior de la pantalla

	random_x_value:							;;|
	call	getRandPosX						;;| ( - /  AF, BC, DE, HL / A )
	ld		barril_x(ix), a					;;|	Cargamos en la X del barril el byte devuelto por el metodo
	call	colision_within_entities		;;| ( IX / AF, BC, DE, HL, IY / A )
	cp		#0								;;|
	jr		z, random_x_value				;;|	Comprobamo si el barril colision con algo en su nueva posicion
	;; -----------------------------------------
	;; No colisiona con nada en nueva Pos
	;; -----------------------------------------
	call	cpct_getRandom_mxor_u8_asm		;;| ( - / AF, BC, DE, HL / L ) 
											;;| Calcula un byte random y lo guarda en L

	ld		a,	l					;;  Pasamos el byte al registro A para poder operar con ella
	and		#0x03					;;  Mascara para tener [0, 3]
	inc		a						;;  Le sumo 1, [1, 4]
	ld		barril_type(ix), a		;; 	Se lo aplicamos al TIPO

	dec		a						;;|			
	cp		#0x00					;;|
	call 	z,	barril_type1		;;| ( IX / HL / - )
									;;| Comprobamos si es del primer tipo

	dec		a						;;|
	cp		#0x00					;;|
	call 	z,	barril_type2		;;| ( IX / HL / - )
									;;|Comprobamos si es del segundo tipo
	
	dec		a						;;|		
	cp		#0x00					;;|
	call 	z,	barril_type3		;;| ( IX / HL / - )
									;;|Comprobamos si es del tercer tipo

	dec		a						;;|
	cp		#0x00					;;|
	call 	z,	barril_type4		;;| ( IX / HL / - )
									;;|Comprobamos si es del cuarto tipo
	ret
;; ======================================================================
;; *Ponemos el sprite del primer tipo
;; ENTRADA: IX
;; MODIFICA: HL
;; SALIDA: --
;; ======================================================================
barril_type1:
	ld	hl, #_pre_roca				;;|
	ld	barril_sprite_h(ix), h		;;|
	ld	barril_sprite_l(ix), l		;;| Cargamos el sprite 1 en el barril
	ret
;; ======================================================================
;; *Ponemos el sprite del segundo tipo
;; ENTRADA: IX
;; MODIFICA: HL
;; SALIDA: --
;; ======================================================================
barril_type2:
	ld	hl, #_pre_car				;;|
	ld	barril_sprite_h(ix), h		;;|
	ld	barril_sprite_l(ix), l		;;| Cargamos el sprite 2 en el barril
	ret
;; ======================================================================
;; *Ponemos el sprite del tercer tipo
;; ENTRADA: IX
;; MODIFICA: HL
;; SALIDA: --
;; ======================================================================
barril_type3:
	ld	hl, #_pre_barriles1			;;|
	ld	barril_sprite_h(ix), h		;;|
	ld	barril_sprite_l(ix), l		;;| Cargamos el sprite 3 en el barril
	ret
;; ======================================================================
;; *Ponemos el sprite del cuarto tipo
;; ENTRADA: IX
;; MODIFICA: HL
;; SALIDA: --
;; ======================================================================
barril_type4:
	ld	hl, #_pre_barriles2			;;|
	ld	barril_sprite_h(ix), h		;;|
	ld	barril_sprite_l(ix), l		;;| Cargamos el sprite 4 en el barril
	ret
;; ======================================================================
;; *Guardamos los valores de X e Y antes del update de barril
;; ENTRADA: IX
;; MODIFICA: A
;; SALIDA: --
;; ======================================================================
save_last_variables:
	ld 	a, barril_x(ix)			;;|
	ld 	barril_last_x(ix), a	;;| Almacenamos la X del barril en last_X

	ld 	a, barril_y(ix)			;;|
	ld 	barril_last_y(ix), a	;;| Almacenamos la Y del barril en last_Y
	ret
;; ======================================================================
;; *Mueve el barril en el eje Y
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
move_barril:
	ld		a, barril_y(ix)		;;|  
	add		a, #2				;;| Almaceamos en A la posicion Y del barril y le añadimos la velocidad
	ld 		d, a				;;| 
	push 	de					;;| Almacenamos en nuevo valor en D y lo anyadimos a la pila

	inc		ix					;;| Cambiamos el eje de la entidad al Y
	ld		b, #197				;;| B - Posicion Pared 2
	ld		c, #6				;;| C - Width Pared 2
	call	colision_pared		;;| ( IX, BC / AF / A )
								;;| Calculamos si colisiona con el fondo de la pantalla

	cp		#0					;;|
	pop		de					;;|	Quitamos DE de la pila (D->Nueva posicion Y)
	jr		z, sin_colision		;;| Comprobamos si hay colision o no
	;; -----------------------------------------
	;; Colision
	;; -----------------------------------------	
	dec 	ix					;;| Volvemos IX a su valor inicial
	call 	barril_init			;;| ( IX / AF, BC, DE, HL, IY / - )
								;;| Actualizamos las variables del barril
	ret	
	sin_colision:
	;; -----------------------------------------
	;; No colision
	;; -----------------------------------------		
	ld 0(ix), d					;; Guardamos en Y la nueva posicion en Y del barril
	ret
