
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

;; ================================================================================
;; ================================================================================
;; PRIVATE DATA
;; ================================================================================
;; ================================================================================

.include "cpctelera.h.s"
.include "collision.h.s"
.include "powerup.h.s"
.include "barril.h.s"
.include "enemy.h.s"
.include "functions.h.s"
.include "hero.h.s"
.include "menu.h.s"
.include "sfx.h.s"
.include "assets.h.s"
.include "macros.h.s"

;; Bullet Definition

;; HERO
defineBullet bullet_1, 0, 0, 1, 2, _sprite_Bullet_Hero_1,	1
defineBullet bullet_2, 0, 0, 1, 2, _sprite_Bullet_Hero_1,	1
defineBullet bullet_3, 0, 0, 1, 2, _sprite_Bullet_Hero_1,	1
defineBullet bullet_4, 0, 0, 1, 2, _sprite_Bullet_Hero_1,	1

;; ENEMY
defineBullet bullet_5, 0, 0, 1, 2, _sprite_Bullet_Enemy,	1
defineBullet bullet_6, 0, 0, 1, 2, _sprite_Bullet_Enemy,	1
defineBullet bullet_7, 0, 0, 1, 2, _sprite_Bullet_Enemy,	1
defineBullet bullet_8, 0, 0, 1, 2, _sprite_Bullet_Enemy,	1
defineBullet bullet_9, 0, 0, 1, 2, _sprite_Bullet_Enemy,	1

bullet_vel_x:	.db	1
bullet_vel_y:	.db	4

clip:		.db	0
max_dist:	.db	25

;; ================================================================================
;; ================================================================================
;; PUBLIC FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================
;; Guardamos las variables de las balas
;; MODIFICA: IX, A
;; ======================
save_last_variables_bullet::
	ld ix, #bullet_1_data
	call save_last_variables

	ld ix, #bullet_2_data
	call save_last_variables

	ld ix, #bullet_3_data
	call save_last_variables

	ld ix, #bullet_4_data
	call save_last_variables

	ld ix, #bullet_5_data
	call save_last_variables

	ld ix, #bullet_6_data
	call save_last_variables

	ld ix, #bullet_7_data
	call save_last_variables

	ld ix, #bullet_8_data
	call save_last_variables

	ld ix, #bullet_9_data
	call save_last_variables

	ret


;; ======================
;; Inicializamos las balas
;; MODIFICA: IX, A
;; ======================
bullet_init_all::
	
	call bullet_erase_all

	ld	hl,		#clip
	ld (hl),	#0

	ld a, #0					;; HERO BULLETS

	ld	ix,	#bullet_1_data		;; Guardamos el puntero a Bullet
	call init_bullet			;; Inicializamos

	ld	ix,	#bullet_2_data
	call init_bullet

	ld	ix,	#bullet_3_data
	call init_bullet

	ld	ix,	#bullet_4_data
	call init_bullet

	ld a, #1					;; ENEMY BULLETS

	ld	ix,	#bullet_5_data
	call init_bullet

	ld	ix,	#bullet_6_data
	call init_bullet

	ld	ix,	#bullet_7_data
	call init_bullet

	ld	ix,	#bullet_8_data
	call init_bullet

	ld	ix,	#bullet_9_data
	call init_bullet

ret

;; ======================
;; Inicializamos el PUP CLIP
;; ======================
init_clip::
	ld	a,	#0
	ld	(clip),	a
ret

;; ======================
;; Borramos la bala
;; MODIFICA: IX, AF, BC, DE, HL
;; ======================
bullet_erase_all::
	
	ld	ix,	#bullet_1_data		;; Guardamos el puntero a Bullet
	call erase_bullet			;; Borramos

	ld	ix,	#bullet_2_data
	call erase_bullet

	ld	ix,	#bullet_3_data
	call erase_bullet

	ld	ix,	#bullet_4_data
	call erase_bullet

	ld	ix,	#bullet_5_data
	call erase_bullet

	ld	ix,	#bullet_6_data
	call erase_bullet

	ld	ix,	#bullet_7_data
	call erase_bullet

	ld	ix,	#bullet_8_data
	call erase_bullet

	ld	ix,	#bullet_9_data
	call erase_bullet

ret

;; ======================
;; Coloreamos la bala
;; MODIFICA: IX, AF, HL, DE, CB
;; ======================
bullet_draw_all::

	ld	ix,	#bullet_1_data		;; Guardamos el puntero a Bullet
	call paint_bullet			;; Pintamos

	ld	ix,	#bullet_2_data
	call paint_bullet

	ld	ix,	#bullet_3_data
	call paint_bullet

	ld	ix,	#bullet_4_data
	call paint_bullet

	ld	ix,	#bullet_5_data
	call paint_bullet

	ld	ix,	#bullet_6_data
	call paint_bullet

	ld	ix,	#bullet_7_data
	call paint_bullet

	ld	ix,	#bullet_8_data
	call paint_bullet

	ld	ix,	#bullet_9_data
	call paint_bullet

ret

;; ======================
;; Movemos la bala y miramos las colisiones
;; ======================
bullet_update_all::

	call cooldown_manager_bullet

	ld	ix,	#bullet_1_data			;; Guardamos el puntero a Bullet
	call update_bullet				;; Hacemos Update
	call bullet_hero_colisions

	ld	ix,	#bullet_2_data
	call update_bullet
	call bullet_hero_colisions

	ld	ix,	#bullet_3_data
	call update_bullet
	call bullet_hero_colisions

	ld	ix,	#bullet_4_data
	call update_bullet
	call bullet_hero_colisions

	ld	ix,	#bullet_5_data
	call update_bullet
	call bullet_enemy_colisions

	ld	ix,	#bullet_6_data
	call update_bullet
	call bullet_enemy_colisions

	ld	ix,	#bullet_7_data
	call update_bullet
	call bullet_enemy_colisions

	ld	ix,	#bullet_8_data
	call update_bullet
	call bullet_enemy_colisions

	ld	ix,	#bullet_9_data
	call update_bullet
	call bullet_enemy_colisions
ret

;; ======================
;; Guardamos la posicion X e Y
;; Entrada: b = X Dir, IX = Pointer to Player
;; ======================
fire_bullet_hero::

	;; Comprobacion
	ld	iy,	#bullet_1_data				;; We save the bullet's memory pos on IY
	ld	a,	bullet_death_cooldown(iy)	;; We save Dir it's shooting on A
	cp	#0
	jr	z,	doShoot						;; If not used, we jump to do the shooting

	ld	iy,	#bullet_2_data
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	z,	doShoot

	ld	iy,	#bullet_3_data
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	z,	doShoot

	ld	iy,	#bullet_4_data
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	nz,	doNotShoot
	
	;; Si encontramos una libre disparamos
	doShoot:
	ld bullet_dir_x(iy), b		;; Bullet X Dir = B = (-1, 0, 1)
	ld bullet_dir_y(iy), #1		;; Bullet Y Dir = 1
	call init_bullet_power_up

	ld	b, 0(ix)				;; Pos X of Player goes into B
	inc	b						;; Add B to center on player
	inc	b
	ld	bullet_x(iy), b			;; Pos X de Bullet is filled

	ld	b, 1(ix)				;; pos Y of Player goes into
	ld	bullet_y(iy), b			;; Pos Y of Player is filled

	;; Si no hay ninguna libre no disparamos
	doNotShoot:

ret

;; ======================
;; Guardamos la posicion X e Y
;; Entrada: IX = Pointer to Enemy -- IY = Pointer to Bullet
;; ======================
fire_bullet_enemy::

	ld bullet_dir_y(iy), #-1	;; Bullet Y Dir = -1

	ld	b, 0(ix)				;; Pos X of Enemy goes into B
	ld	bullet_x(iy), b			;; Pos X de Bullet is filled
	
	ld	b,	1(ix)				;; pos Y of Player goes into B
	ld	a,	b
	add	a,	#10
	ld	b,	a
	ld	bullet_y(iy), b			;; Pos Y of Player is filled

ret

;; ======================
;; Apply Power Up
;; ENTRADA: IY = Power Up, IX = PLAYER
;; ======================
apply_power_up::

	push	ix				;; Nos guardamos IX
	call rellenar_balas
	call power_up_init
	call play_obtain_pup_sfx
	pop		ix				;; Recuperamos IX

	ld	hl,		#clip		;;|
	ld	(hl),	#10			;;| We get the ammo back to MAX

ret

;; ================================================================================
;; ================================================================================
;; PRIVATE FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================================================================
;; Inicializa una Bullet
;; ENTRADA: IX --> Bullet, A ---> 0 if Hero, else if Enemy
;; ======================================================================
init_bullet:
	
	ld		bullet_x(ix),					#0
	ld		bullet_y(ix),					#0
	ld		bullet_dir_x(ix),				#0
	ld		bullet_dir_y(ix),				#0
	ld		bullet_damage(ix),				#1
	ld		bullet_dist(ix),				#0
	ld		bullet_death_cooldown(ix),		#0
	
	ld		bullet_sprite_l(ix),	#_sprite_Bullet_Hero_1
	cp		#0	;; |
	ret 	 z	;; | If a = 0, Hero Bullet so we RETURN
	ld		bullet_sprite_l(ix),	#_sprite_Bullet_Enemy

	ret

;; ======================================================================
;; Reduce el cooldown
;; ======================================================================
cooldown_manager_bullet::
	
	ld	iy,	#bullet_1_data
	ld	a,	bullet_dir_y(iy)
	cp	#0
	jr	nz,	next1
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	z,	next1
	dec	a
	ld	bullet_death_cooldown(iy), a

	next1:
	ld	iy,	#bullet_2_data
	ld	a,	bullet_dir_y(iy)
	cp	#0
	jr	nz,	next2
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	z,	next2
	dec	a
	ld	bullet_death_cooldown(iy), a

	next2:
	ld	iy,	#bullet_3_data
	ld	a,	bullet_dir_y(iy)
	cp	#0
	jr	nz,	next3
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	z,	next3
	dec	a
	ld	bullet_death_cooldown(iy), a

	next3:
	ld	iy,	#bullet_4_data
	ld	a,	bullet_dir_y(iy)
	cp	#0
	jr	nz,	next4
	ld	a,	bullet_death_cooldown(iy)
	cp	#0
	jr	z,	next4
	dec	a
	ld	bullet_death_cooldown(iy), a

	next4:

ret

;; ======================================================================
;; Inicializa una Bullet con POWERUP
;; ENTRADA: IY --> Bullet
;; MODIFICA: HL, A
;; ======================================================================
init_bullet_power_up:

	ld	hl,	#clip					;; HL = CLIP

	ld	a,	(hl)					;;|
	cp	#0							;;|
	jr	z,	init_bullet_no_power_up	;;| If we do not have ammo, we initialize it as normal bullet

	dec	a							;;|
	ld	(hl),	a					;;| If we have ammo, we decrement it

	ld	bullet_w(iy),					#2							;;|
	ld	bullet_h(iy),					#8							;;|	
	ld	bullet_sprite_l(iy),			#_sprite_Bullet_Hero_2		;;|
	ld	bullet_death_cooldown(iy),		#100							;;|
	ld	bullet_damage(iy),				#2							;;| We put the POWERUP DATA
	
	push ix
	push iy
	call erase_bala_gui
	call play_Shoot_pup_sfx
	pop iy
	pop ix
	
	ret

	init_bullet_no_power_up:
	ld	bullet_w(iy),					#1							;;|
	ld	bullet_h(iy),					#2							;;|	
	ld	bullet_sprite_l(iy),			#_sprite_Bullet_Hero_1		;;|
	ld	bullet_death_cooldown(iy),		#100							;;|
	ld	bullet_damage(iy),				#1							;;| We put the DEFAULT DATA

	push ix
	push iy
	call play_Shoot_sfx
	pop iy
	pop ix
	ret

;; ======================
;; Guardamos los ultimos valores XY antes del update
;; ENTRADA: IX = Puntero a Bullet
;; MODIFICA: A
;; ======================
save_last_variables:
	ld a, bullet_x(ix)
	ld bullet_last_x(ix), a

	ld a, bullet_y(ix)
	ld bullet_last_y(ix), a

	ld a, bullet_dir_y(ix)
	ld bullet_last_dir_y(ix), a
ret


;; ======================
;; Update any bullet
;; ENTRADA: IX = Puntero a Bullet
;; MODIFICA: AF, BC, HL
;; ======================
update_bullet:

	;; Guardamos los valores XY antes del update
	;; call save_last_variables

	;; Check if activated
	ld	 a,	bullet_dir_y(ix)		;; A = Bullet Y Dir
	cp	#0							;; We check if activated
	ret	 z							;; If we're not activated we end the update
	call move_bullet_y				;; We move along Y axis

	;; Check distance (If activated)
	ld	a,	bullet_dist(ix)			;; A = Distance
	inc	bullet_dist(ix)				;; We increment the distanc
	ld	hl,	#max_dist				;;|
	cp 	(hl)						;;| We check against the max distance
	call z, deactivateBullet		;; If we've reached max distance we deactivate

	;; Check if Moving X
	ld	 a,	bullet_dir_x(ix)		;; A = Bullet X Dir
	cp	#0							;; We check if moving in axis X
	jr	 z,	endUpdateFunction		;; If we're not moving along axis X we end the update
	call move_bullet_x				;; We move along X axis

	endUpdateFunction:

ret

;; ======================
;; Mueve la bala en Y
;; ENTRADA: IX = Pointer to Bullet
;; ======================
move_bullet_y:

	;; We save out velocity
	ld	a,	(#bullet_vel_y)
	ld 	b,	a

	;; Check whose bullet we are
	ld	 a,	bullet_dir_y(ix)		;; A = Bullet Y Dir
	cp	#1							;; Is a Player Bullet
	jr	 z, update_isPlayer

		;; Is an Enemy Bullet
		ld	a,	bullet_y(ix)		;; We point to the Y position
		add b						;; We recalculate Y position
		jr	check_y_boundaries		;; We jump to not overwritte

	;; Is a Player Bullet
	update_isPlayer:
	ld	a,	bullet_y(ix)			;; We point to the Y position
	sub b							;; We recalculate Y position

	check_y_boundaries:				;; Now we check if we're touching the boundaries
	ld	bullet_y(ix),	a			;; If we have not deactivated, we load the new pos
	inc	ix
	ld	b,	#195					;; Cargamos la POSICION de la PARED
	ld	c,	#10						;; Cargamos el ANCHO de la PARED
	call colision_pared
	dec ix
	cp	#1
	call z, deactivateBullet

ret

;; ======================
;; Mueve la bala en X
;; ENTRADA: IX = Pointer to Bullet
;; MODIFICA: AF, HL
;; ======================
move_bullet_x:

	;; We save out velocity
	ld	a,	(#bullet_vel_x)
	ld 	b,	a

	;; Check direction
	ld	 a,	bullet_dir_x(ix)		;; A = Bullet X Dir
	cp	#1							;; Moving RIGHT
	jr	 z, move_bullet_right

		;; Moving LEFT
		ld	a,	bullet_x(ix)		;; We point to the Y position
		sub b						;; We recalculate Y position
		jr	check_x_boundaries			;; We jump to not overwritte

	;; Moving RIGHT
	move_bullet_right:
	ld	a,	bullet_x(ix)		;; We point to the Y position
	add b						;; We recalculate Y position

	check_x_boundaries:				;; Now we check if we're touching the boundaries
	cp #19							;; |
	call z, deactivateBullet		;; | Touching Left Bundarie
	cp #79							;; |
	call z, deactivateBullet		;; | Touching Right Bundarie
	ld bullet_x(ix), a				;; If we're not touching boundaries we save the new position

ret

;; ======================
;; Desactiva la bala
;; ENTRADA: IX = Putero a Bullet
;; ======================
deactivateBullet::

	;; We set the X = 0
	ld bullet_x(ix), #0

	;; We set the Y = 0
	ld bullet_y(ix), #0

	;; We set the Dir X = 0
	ld bullet_dir_x(ix), #0

	;; We set the Dir X = 0
	ld bullet_dir_y(ix), #0

	;; We set the distance to 0
	ld bullet_dist(ix), #0

	ld bullet_death_cooldown(ix), #2	

ret

;; ======================
;; Colorea la bala si esta activa
;; ENTRADA: IX = Putero a Bullet
;; MODIFICA: AF, HL, DE, CB
;; ======================
paint_bullet:
	ld	 a,	bullet_dir_y(ix)	;; We get the Y Direction on A
	cp	 a,	#0					;; |
	call nz, paint_entity		;; | A != 0, It means activated so we paint
ret

;; ======================
;; Borra la bala si esta activa
;; ENTRADA: IX = Putero a Bullet
;; MODIFICA: AF, HL, DE, CB
;; ======================
erase_bullet:
	ld	 a,	bullet_last_dir_y(ix)	;; We get the Y Direction on A
	cp	 a,	#0					;; |
	call nz, erase_entity		;; | A != 0, It means activated so we erase
ret

;; =========================
;; Colision Balas Heroe
;; ENTRADA: IY -> Entity_X. IX --> Bullet
;; MODIFICA: 
;; =========================
colision_bullet:
		
	call colision_entidad_total
	cp #0
	ld a, #0				;; BOOLEAN FALSE
	ret z					;; ---------------- No hay colision
							;; ---------------- Colision
	call deactivateBullet	;; Desactivar la bala
	ld a, #1				;; BOOLEAN TRUE

ret

;; =========================
;; Colision Balas Enemigos Entidades
;; ENTRADA: IY -> Entity_X
;; MODIFICA: 
;; =========================
bullet_enemy_colisions:
	bullet_enemy_entidad_1:
	ld iy, #barril1_variables
	call colision_bullet
	cp #1
	jr nz, bullet_enemy_entidad_2
	;; call_______
	ret

	bullet_enemy_entidad_2:
	ld iy, #barril2_variables
	call colision_bullet
	cp #1
	jr nz, bullet_enemy_entidad_3
	;; call_______
	ret

	bullet_enemy_entidad_3:
	ld iy, #hero_variables
	call colision_bullet
	cp #1
	jr nz, bullet_enemy_ninguna_entidad
	call dec_life
	ret

	bullet_enemy_ninguna_entidad:

	ret


;; =========================
;; Colision Balas Heroe Entidades
;; ENTRADA: IY -> Entity_X
;; MODIFICA: 
;; =========================
bullet_hero_colisions:
	bullet_hero_entidad_1:
	ld iy, #barril1_variables
	call colision_bullet
	cp #1
	jr nz, bullet_hero_entidad_2
	;;call _______
	ret

	bullet_hero_entidad_2:
	ld iy, #barril2_variables
	call colision_bullet
	cp #1
	jr nz, bullet_hero_entidad_3
	;;call _______
	ret

	bullet_hero_entidad_3:
	ld iy, #enemy1_data
	call colision_bullet
	cp #1
	jr nz, bullet_hero_entidad_4
	call enemy_bullet
	ret


	bullet_hero_entidad_4:
	ld iy, #enemy2_data
	call colision_bullet
	cp #1
	jr nz, bullet_hero_entidad_5
	call enemy_bullet
	ret


	bullet_hero_entidad_5:
	ld iy, #enemy3_data
	call colision_bullet
	cp #1
	jr nz, bullet_hero_entidad_6
	call enemy_bullet
	ret

	bullet_hero_entidad_6:
	ld iy, #enemy4_data
	call colision_bullet
	cp #1
	jr nz, bullet_hero_entidad_7
	call enemy_bullet
	ret

	bullet_hero_entidad_7:
	ld iy, #enemy5_data
	call colision_bullet
	cp #1
	jr nz, bullet_hero_ninguna_entidad
	call enemy_bullet
	ret

	bullet_hero_ninguna_entidad:

ret