
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

.include "barril.h.s"
.include "enemy.h.s"
.include "hero.h.s"
.include "piezas.h.s"

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PUBLIC FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Comprueba con cuantos enemigos colisiona a la vez
;; Lo perfeto es que colisione con uno ( el mismo )
;; ENTRADA: IX --> Enemigo
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: A
;;		A == 1	->	Colisiona con el mismo
;;		A == 0	->	Colisiona con otra entidad
;; ======================================================================
colision_within_enemys::
	ld 		e, #0						;;  Reseteamos el registro E a 0

	ld 		iy, #enemy1_data			;;|
	call 	colision_entidad_total		;;| ( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;|
	ld 		iy, #enemy2_data			;;|
	call 	colision_entidad_total		;;|	( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;|
	ld 		iy, #enemy3_data			;;| ...
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;|
										;;|
	ld 		iy, #enemy4_data			;;|
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;| Vamos haciendo las comprobaciones de colision entre la entidad1
										;;| y todas las demas entidades del juego
	ld 		iy, #enemy5_data			;;|
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;|
										;;|
	ld 		iy, #barril1_variables		;;|
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;|
										;;| ...
	ld		iy, #barril2_variables		;;|
	call 	colision_entidad_total		;;| ( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;| Comprobamos la colision entre las entidades y le sumamos la salida al registro E
										;;| Se suma 1 si hay colision, Se suma 0 si no hay colision

	ld 		a, e						;;|
	cp 		#1							;;|
	ld 		a, #0						;;| Cargamos el valor de SALIDA == 0
	ret 	nz							;;| Comprobamos si colisiona con alguna entidad
	;; -----------------------------------------
	;; Solo colision consigo mismo
	;; -----------------------------------------
	ld 		a, #1						;; Cargamos el valor de SALIDA == 1
	ret
;; ======================================================================
;; *Comprueba con cuantos elementos colisiona a la vez
;; Lo perfeto es que colisione con uno ( el mismo )
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: A
;;		A == 1	->	Colisiona con el mismo
;;		A == 0	->	Colisiona con otra entidad
;; ======================================================================
colision_within_entities::
	ld 		e, #0						;;  Reseteamos el registro E a 0

	ld 		iy, #enemy1_data			;;|
	call 	colision_entidad_total		;;| ( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;|
	ld 		iy, #enemy2_data			;;|
	call 	colision_entidad_total		;;| ( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;|
	ld 		iy, #enemy3_data			;;|
	call 	colision_entidad_total		;;| ( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;| 
	ld 		iy, #enemy4_data			;;| ...
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;| 
										;;| 
	ld 		iy, #enemy5_data			;;|
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;|
										;;|
	ld 		iy, #barril1_variables		;;|
	call 	colision_entidad_total		;;| Vamos haciendo las comprobaciones de colision entre la entidad1
	call 	sumar_colisiones			;;| y todas las demas entidades del juego
										;;|
	ld 		iy, #barril2_variables		;;|
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;|
										;;|
	ld 		iy, #hero_variables			;;|
	call 	colision_entidad_total		;;|
	call 	sumar_colisiones			;;|
										;;| ...
	ld 		iy, #pieza1_variables		;;|
	call 	colision_entidad_total		;;| ( IX, IY / AF / A )
	call 	sumar_colisiones			;;| ( A / E / E )
										;;| Comprobamos la colision entre las entidades y le sumamos la salida al registro E
										;;| Se suma 1 si hay colision, Se suma 0 si no hay colision

	ld 		a, e						;;|
	cp 		#1							;;|
	ld 		a, #0						;;| Cargamos el valor de SALIDA == 0
	ret 	nz							;;| Comprobamos si colisiona con alguna entidad
	;; -----------------------------------------
	;; Solo colision consigo mismo
	;; -----------------------------------------
	ld 		a, #1						;; Cargamos el valor de SALIDA == 1
	ret
;; ======================================================================
;; Colision de las entidades entre ellas en los dos ejes 
;; ENTRADA: IX, IY
;;		IX 	->	Entidad1 
;;		IY 	-> 	Entidad2
;; MODIFICA: AF
;; SALIDA: A
;;		A == 1	-> Colision
;;		A == 0	-> Sin Colision
;; =====================================================================
colision_entidad_total::
	call 	colision_entidad		;;| ( IX, IY, / AF / A )
									;;| Comprobamos si colisionan en el ejeX
	cp 		#0						;;|
	ld 		a, #0					;;| Cargamos el valor de SALIDA == 0
	ret 	z						;;| En el caso que de 0 es que no hay colision

	inc 	ix						;;|			
	inc 	iy						;;| Incrementamos IX e IY para comrobar la colision en Y

	call 	colision_entidad		;;| ( IX, IY / AF / A )
									;;| Comprobamos si colisionan en el ejeY
	dec 	ix						;;|
	dec 	iy						;;| Volvemos a poner los valores iniciales de las variables
	cp 		#0						;;|
	ld 		a, #0					;;| Cargamos el valor de SALIDA == 0
	ret 	z						;;| En el caso que de 0 es que no hay colision
	;; -----------------------------------------
	;;Colision
	;; -----------------------------------------
	ld 		a, #1					;; Cargamos el valor de SALIDA == 1
	ret
;; ======================================================================
;; *Colision de las entidades entre ellas
;;
;; 	!! Hay que pasar el mismo eje para las dos entidades
;;	!! EjeX -> (IX) == 0(IX)
;;	!! EjeY -> (IX) == 1(IX)
;;
;; ENTRADA: IX, IY
;;		IX 	-> 	Entidad1
;;		IY 	-> 	Entidad2
;; MODIFICA: AF
;; SALIDA: A
;;		A == 1	->	Colision
;;		A == 0	-> 	Sin Colision
;; =====================================================================
colision_entidad::
	;; -----------------------------------------
	;; Posicion1 + Ancho1 - Posicion2 - 1 < 0
	;; -----------------------------------------
	ld 		a, 0(ix)				;;| Cargamos en A la posicion de la Entidad1	
	add 	a, 2(ix)				;;| Le sumamos a la posicion el ancho de la Entidad1
	sub 	0(iy)					;;| Le restamos la posicion de la Entidad2
	sub 	#1						;;| Le restamos 1 para solamente tener que comprobar si es menor que 0
	jp		m, sin_colision			;;| En el caso de que sea negativo es que no ha habido colision
	;; -----------------------------------------
	;; Posicion2 + Ancho2 - Posicion1 - 1 < 0	
	;; -----------------------------------------
	ld		a, 0(iy)				;;| Cargamos en A la posicion de la Entidad2		
	add 	a, 2(iy)				;;| Le sumamos a la posicion el ancho de la Entidad2
	sub 	0(ix)					;;| Le restamos la posicion de la Entidad1
	sub 	#1						;;| Le restamos 1 para solamente tener que comprobar si es menor que 0
	jp 		m, sin_colision			;;| En el caso de que sea negativo es que no ha habido colision
	;; -----------------------------------------
	;;Colision
	;; -----------------------------------------
	ld 		a, #1					;; Cargamos el valor de SALIDA == 1
	ret
;; ======================================================================
;; *Colision entre objeto y pared
;;
;; 	!! Para cada eje hay que seguir la siguiente igualacion
;;	!! EjeX -> (IX) == 0(IX)
;;	!! EjeY -> (IX) == 1(IX)
;;
;; ENTRADA: IX, BC
;;		IX 	->	Puntero a la posicion X/Y de la entidad1
;;		B  	->  B posicion objeto 2  
;;		C  	->	C ancho objeto 2   
;; MODIFICA: AF 
;; SALIDA: A
;;		A == 1	-> Colision
;;		A == 0	-> Sin Colision		
;; =====================================================================
colision_pared::
	;; -----------------------------------------	
	;; Posicion1 + Ancho1 - Posicion2 - 1 < 0
	;; -----------------------------------------		
	ld 		a, 0(ix)				;;|	Cargamos en A la posicion del objeto			
	add 	a, 2(ix)				;;| Le sumamos a la posicion el ancho de este
	sub 	b						;;| Le restamos la posicion de la pared
	sub 	#1						;;| Le restamos 1 para solamente tener que comprobar si es menor que 0
	jp		m, sin_colision			;;| En el caso de que sea negativo es que no ha habido colision
	;; -----------------------------------------		
	;; Posicion2 + Ancho2 - Posicion1 - 1 < 0
	;; -----------------------------------------
	ld 		a, b					;;| Cargamos en A la posicion de la pared		
	add 	a, c					;;| Le sumamos a la posicion el ancho de la pared
	sub 	0(ix)					;;| Le restamos la posicion del objeto
	sub 	#1						;;| Le restamos 1 para solamente tener que comprobar si es menor que 0
	jp 		m, sin_colision			;;| En el caso de que sea negativo es que no ha habido colision
	;; -----------------------------------------
	;; Colision
	;; -----------------------------------------	
	ld 		a, #1					;; Cargamos el valor de SALIDA == 1
	ret
	;; -----------------------------------------
	;; Sin Colision
	;; -----------------------------------------	
	sin_colision:
	ld a, #0						;;  Cargamos el valor de SALIDA == 2
	ret

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PRIVATE FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Suma al registro E el valor que haya en A
;; ENTRADA: A
;; MODIFICA: E
;; SALIDA: E
;; ======================================================================
sumar_colisiones:
	add 	a, e	;;|	
	ld 		e, a	;;| Suma al registro E el valor que haya en A
	ret
