
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

.include "cpctelera.h.s"
.include "collision.h.s"
.include "bullet.h.s"
.include "hero.h.s"
.include "powerup.h.s"
.include "functions.h.s"
.include "niveles.h.s"
.include "assets.h.s"
.include "sfx.h.s"
.include "macros.h.s"


.globl bullet_5_data
.globl bullet_6_data
.globl bullet_7_data
.globl bullet_8_data
.globl bullet_9_data

;; ================================================================================
;; ================================================================================
;; PRIVATE DATA
;; ================================================================================
;; ================================================================================


;; Enemy Definition
	defineEnemy enemy1, 0,  15, 4, 10, 1, _sprite_Enemy, bullet_5_data
	defineEnemy enemy2, 0,  30, 4, 10, 1, _sprite_Enemy, bullet_6_data
	defineEnemy enemy3, 0,  45, 4, 10, 1, _sprite_Enemy, bullet_7_data
	defineEnemy enemy4,	0,  60, 4, 10, 0, _sprite_Enemy, bullet_8_data
	defineEnemy enemy5, 0,  75, 4, 10, 0, _sprite_Enemy, bullet_9_data

;; Enemy animation
enemy_animation:
	enemy_anim_0: 		.dw	#_sprite_Enemy
	enemy_anim_1:		.dw #_sprite_Enemya
	enemy_anim_2: 		.dw	#_sprite_Enemy
	enemy_anim_3:		.dw #_sprite_Enemyb

enemy_vel_x:			.db	#4
enemy_vel_y:			.db	#2
first_init:				.db #1
update_cooldown:		.db #10

;; Variables de actualizacion de vertical y horizontal
update_max_cooldown:	.db #10			;; -> enemy_init_movement:
update_max_h:			.db #8

;; ================================================================================
;; ================================================================================
;; PUBLIC FUNCTIONS
;; ================================================================================
;; ================================================================================


;; ======================================================================
;; Vuelve a la version inicial de los enemigos
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
enemy_init_movement::
	ld hl, #update_max_cooldown
	ld (hl), #10
	ld hl, #update_max_h
	ld (hl), #8
	ret

;; ======================================================================
;; Activar solamentre 3 enemigos al empezar el juego
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
enemy_desactive::
	ld ix, #enemy4_data
	ld enemy_dir_y(ix), #0
	ld enemy_last_dir_y(ix), #0
	ld enemy_x(ix), #0
	ld enemy_y(ix), #60

	ld ix, #enemy5_data
	ld enemy_dir_y(ix), #0
	ld enemy_last_dir_y(ix), #0
	ld enemy_x(ix), #0
	ld enemy_y(ix), #75

	ret
;; ======================================================================
;; *Incrementa las variables de IA de los enemigos ( contadores de actualizacion )
;; ENTRADA: -- 
;; MODIFICA: AF
;; SALIDA: --
;; ======================================================================
enemy_buff::
	buff_x:
	ld 		a, (update_max_cooldown)		;;|	Cargamos el cooldown para mover vertical	
	cp 		#1								;;| 
	jr 		z, buff_y						;;| Comprobamos si esta en el valor minimo
	dec 	a								;;| 
	ld 		(update_max_cooldown), a		;;| En el caso de que no este al minimo lo decrementamos

	buff_y:
	ld 		a, (update_max_h)				;;| Cargamos el cooldown para mover horizontal
	cp 		#2								;;|
	ret 	z								;;| Comprobamos si esta en el valor minimo
	dec 	a								;;|
	ld 		(update_max_h), a				;;| En el caso de que no ese al minimo lo decrementamos

	ret
;; ======================================================================
;; Inicializamos los enemigos al empezar la partida
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
enemy_init_all::

	ld	hl,	#first_init
	ld	(hl),	#1

	ld		ix, #enemy1_data		;; We load the first Enemy
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, enemy_init			;; we move it

	ld		ix, #enemy2_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, enemy_init			;; we move it

	ld		ix, #enemy3_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, enemy_init			;; we move it

	ld		ix, #enemy4_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, enemy_init			;; we move it

	ld 		ix, #enemy5_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, enemy_init			;; we move it

	ld	hl,	#first_init
	ld	(hl),	#0
	ret

;; ======================================================================
;; Update del enemigo en el que se movera e igual dispara
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
enemy_update::

	;; Move X has its own cooldown
	call enemy_move_horizontal

	;; We check if we have to shoot / move Y
	ld	a,	(update_cooldown)	;; Cargamos el COOLDOWN
	dec	a						;; Le restamos uno
	ld	(update_cooldown),	a	;; Lo volvemos a guardar
	cp	#0						;; Comprobamos que sea 0
	ret nz						;; Si Colldown != 0, volvemos

	;; Si El cooldown ha acabado, lo ponemos de nuevo
	ld	a,	(update_max_cooldown)
	ld	(update_cooldown),	a

	;; We move Y
	call enemy_move_vertical

	;; We shoot
	ld		ix, #enemy1_data		;; We load the first Enemy
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, check_shoot				;; We Shoot
	call 	nz, enemy_update_animation

	ld		ix, #enemy2_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, check_shoot
	call 	nz, enemy_update_animation

	ld		ix, #enemy3_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, check_shoot
	call 	nz, enemy_update_animation

	ld		ix, #enemy4_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call	nz, check_shoot
	call 	nz, enemy_update_animation

	ld 		ix, #enemy5_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, check_shoot
	call 	nz, enemy_update_animation

	ret

;; ======================================================================
;; Guarda las variables del frame anterior
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
save_last_variables_enemy::

	;; We move the enemy
	ld		ix, #enemy1_data		
	call	save_last_variables				

	ld		ix, #enemy2_data
	call	save_last_variables

	ld		ix, #enemy3_data
	call	save_last_variables

	ld		ix, #enemy4_data
	call	save_last_variables

	ld		ix, #enemy5_data
	call	save_last_variables

	ret

;; ======================================================================
;; Borra al enemigo
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
enemy_erase::
	ld	ix, #enemy1_data		;; We load the first Enemy
	ld	a, enemy_last_dir_y(ix)	;;| 
	cp	#0						;;|
	call nz, erase_entity		;;| If Enemy Y Dir != 0, it's activated and we do erase it

	ld	ix, #enemy2_data
	ld	a, enemy_last_dir_y(ix)
	cp	#0
	call nz, erase_entity

	ld	ix, #enemy3_data
	ld	a, enemy_last_dir_y(ix)
	cp	#0
	call nz, erase_entity

	ld	ix, #enemy4_data
	ld	a, enemy_last_dir_y(ix)
	cp	#0
	call nz, erase_entity

	ld	ix, #enemy5_data
	ld	a, enemy_last_dir_y(ix)
	cp	#0
	call nz, erase_entity

	ret

;; ======================================================================
;; Pinta al enemigo
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
enemy_draw::
	ld	ix, #enemy1_data		;; We load the first Enemy
	ld	a, enemy_dir_y(ix)		;;| 
	cp	#0						;;|
	call nz, paint_entity		;;| If Enemy Y Dir != 0, it's activated and we do paint it


	ld	ix, #enemy2_data
	ld	a, enemy_dir_y(ix)
	cp	#0
	call nz, paint_entity

	ld	ix, #enemy3_data
	ld	a, enemy_dir_y(ix)
	cp	#0
	call nz, paint_entity

	ld	ix, #enemy4_data
	ld	a, enemy_dir_y(ix)
	cp	#0
	call nz, paint_entity

	ld	ix, #enemy5_data
	ld	a, enemy_dir_y(ix)
	cp	#0
	call nz, paint_entity

	ret

;; ======================================================================
;; Comprueba si los enemigos estan colisionando con las balas
;; ENTRADA: IY --> Enemigo
;; MODIFICA: IX, AF, BC, DE, HL
;; ======================================================================
enemy_bullet::

	

	ld a,  enemy_life(iy)
	sub 11(ix)
	ld enemy_life(iy), a	;; Le restamos un punto de vida al enemigo
	cp #1					;; En el caso de que tenga uno de vida o mas volvera por el ret
							;; En el caso contrario dara un valor negativo

	push af						;; Nos guardamos el resultado
	call p, play_enemy_hit_sfx	;; Llamamos al SFX de restar vida si no morimos
	pop af						;; Lo volvemos a sacar
	ret p						;; Enemigo no muerto

	push iy					;;|
	pop ix					;;| Pasamos el valor iy al ix

	call hero_add_point		;; We add a point to the hero
	call try_spawning_pup	;; We try to spawn power up
	call erase_entity		;;|
	call enemy_init			;;| Matamos e reinicializamos el enemigo
	call play_enemy_death_sfx ;; Llamamos al SFX de matar enemigo

	call dec_enemy_nivel	;;| CAUTION- Modifica IX

	ret

enemy_move_vertical::

	ld 		ix, #enemy1_data
	ld		a, enemy_dir_y(ix)
	cp 		#0
	call 	nz, move_enemy

	ld ix, #enemy2_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, move_enemy

	ld ix, #enemy3_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, move_enemy

	ld ix, #enemy4_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, move_enemy

	ld ix, #enemy5_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, move_enemy

	ret

enemy_move_horizontal::

	ld 		ix, #enemy1_data
	ld		a, enemy_dir_y(ix)
	cp 		#0
	call 	nz, enemy_move_h

	ld ix, #enemy2_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, enemy_move_h

	ld ix, #enemy3_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, enemy_move_h

	ld ix, #enemy4_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, enemy_move_h

	ld ix, #enemy5_data
	ld		a, enemy_dir_y(ix)
	cp		#0
	call 	nz, enemy_move_h

	ret

;; ======================================================================
;; Move Right Left
;; ENTRADA: IX-> Enemy
;; MODIFICA:
;; ======================================================================
enemy_move_h::
	ld a, enemy_n_target(ix)
	sub #1
	ld enemy_n_target(ix), a
	ret nz
	ld a, (update_max_h)
	ld enemy_n_target(ix), a

	ld a, enemy_target(ix)
	cp #0
	jr nz, move_horizontal
	call enemy_new_target

	move_horizontal:
	ld a, enemy_target(ix)
	cp #0
	jp m, inc_target
	;;------------------------
	dec_target:					;; Cargamos en modifier la direccion
	ld hl, #modifier
	ld (hl), #-1
	jr save_variables
	;;;;------------------------
	inc_target:					;; Cargamos en modifier la direction
	ld hl, #modifier
	ld (hl), #1
	
	save_variables:

	modifier = .+1				;; Variable en el codigo
	add a, #1					;; El #1 es la variable modifier
	ld enemy_target(ix), a		;; Le quitamos uno al target para que se aproxime al 0

	ld a, (modifier)
	ld b, a
	ld a, enemy_x(ix)
	sub b
	ld enemy_x(ix), a		;; Movemos al enemigo y en el caso de que colisione corregimos el movimiento

	cp #19					;; Compruebo si con la suma se sale de la pantalla por la izquierda
	jr z, correct_colision
	cp #77					;; Compruebo si con la suma se sale de la pantalla por la derecha
	jr z, correct_colision


	call colision_within_enemys
	cp #1
	ret z

	correct_colision:
	call enemy_change_direction
	ld a, (modifier)
	ld b, a
	ld a, enemy_x(ix)
	add a, b
	ld enemy_x(ix), a


	ret


;; ======================================================================
;; Update del sprite del heroe
;; MODIFICA: AF, BC, DE, HL
;; ENTRADA: IX ==> ENEMY
;; ======================================================================
enemy_update_animation::

	ld	 a,	enemy_dir_y(ix)				;;| 
	cp	#0								;;|
	ret z								;;| If 0, it's deactivated and we return


	ld 		a, enemy_anim_index(ix)
	inc 	a
	ld 		enemy_anim_index(ix), a
	cp 		#4
	jr		nz, cargar_sprite_nuevo
	
		ld		a, #0
		ld 		enemy_anim_index(ix), a

	cargar_sprite_nuevo:
	call	load_new_sprite

ret

;; ================================================================================
;; ================================================================================
;; PRIVATE FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================================================================
;; Change direction
;; ENTRADA: IX-> Enemy
;; MODIFICA:
;; ======================================================================

enemy_change_direction:
	ld a, enemy_target(ix)
	ld b, a
	sub b
	sub b
	ld enemy_target(ix), a


	ret

;; ======================================================================
;; Calcular el nuevo target
;; ENTRADA: IX-> Enemy
;; MODIFICA:
;; ======================================================================

enemy_new_target:
	ld iy, #hero_variables
	ld a, 0(iy)
	sub 0(ix)
	ld enemy_target(ix), a
	ret

;; ======================================================================
;; 12'5% of spawning power up
;; ENTRADA: 
;; MODIFICA:
;; ======================================================================
try_spawning_pup:
	call	cpct_getRandom_mxor_u8_asm		;; We fetch a rnd [0, 7]
	ld		a,	l							;; A = L = Rnd Num
	ld		b,	#0x03						;; Mascara para coger 2 bits
	and		b								;; A = [0, 7]

	cp	#0									;; Check against 0, (12'5% of spawning)
	call z, power_up_spawn					;; Spawneamos el power up
	ret

;; ======================================================================
;; *Inicializa las variables del enemy
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
enemy_init::
	ld		enemy_y(ix),		#0			;; Movemos el enemigo a la parte superior de la pantalla

	random_x_value:							;;|
	call	getRandPosX						;;| ( - /  AF, BC, DE, HL / A )
	ld		enemy_x(ix),		a			;;| Cargamos en la X del enemigo el byte devuelto por el metodo
	call	colision_within_entities		;;| ( IX / AF, BC, DE, HL, IY / A )
	cp		#0								;;|
	jr		z,	random_x_value				;;| Comprobamo si el enemigo colision con algo en su nueva posicion
	;; -----------------------------------------
	;; No colisiona con nada en nueva Pos
	;; -----------------------------------------
	ld		a, #0							;;|
	ld		enemy_anim_index(ix), a			;;| Actualizamos el indice de la animacion
	call	load_new_sprite					;;| ( A / BC, D, HL / - )
											;;| Cargamos el sprite correspondiente al enemigo
	ld		enemy_dir_x(ix),	#0			;;|
	ld		enemy_dir_y(ix),	#1			;;|
	ld		enemy_life(ix),		#2			;;|
	ld		enemy_target(ix),	#0			;;|
	ld 		enemy_n_target(ix),	#1			;;| Cargamos las variables a los valores por defecto

	ret
;; ======================================================================
;; Guardamos los ultimos valores XY antes del update
;; ENTRADA: A
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
save_last_variables:

	ld a, enemy_x(ix)
	ld enemy_last_x(ix), a

	ld a, enemy_y(ix)
	ld enemy_last_y(ix), a

	ld a, enemy_dir_y(ix)
	ld enemy_last_dir_y(ix), a

	ret

;; ======================================================================
;; Mira si tiene que disparar
;; ENTRADA: IX -> Enemigo seleccionado
;; MODIFICA:
;; ======================================================================
check_shoot:

	ld	iy,	#hero_variables	;; Cargamos en IY el Jugador

	;; CHECK Y
	ld	a,	enemy_y(ix)		;;|
	ld	b,	enemy_y(iy)		;;|
	sub b					;;| Y Offset
	jp	p, #doNotShoot		;; Offset < 0 = No Disparamos (Enemigo ya ha pasado a Jugador)
	cp	#25					;; Comparamos Offset con Dist Maxima
	jp	p, #doNotShoot		;; Offset > Distancia Minima (No disparamos)

	;; CHECK X
	ld	a,	0(ix)			;;|
	add a,	2(ix)			;;|
	add	a,	#8				;;|
	sub 0(iy)				;;|
	sub #1					;;| Posicion1 + Ancho1 (+ OFFSET) - Posicion2 - 1 < 0
	jp	m, doNotShoot

	ld	a,	0(iy)			;;|
	add	a,	2(iy)			;;|
	add	a,	#8				;;|
	sub	0(ix)				;;|
	sub	#1					;;| Posicion2 + Ancho2 (+ OFFSET) - Posicion1 - 1 < 0	
	jp	m, doNotShoot

	doShoot:
	call	enemy_shoot
	doNotShoot:
	ret

;; ======================================================================
;; Disparo una bala
;; ENTRADA: IX = Enemy
;; MODIFICA:
;; ======================================================================
enemy_shoot:

	ld		a,	13(ix)					;;|
	ld		l,	a						;;|
	ld		a,	14(ix)					;;|
	ld		h,	a						;;| HL --> Bullet Pointer
	
	push	hl							;;|
	pop		iy							;;| Pasamos de HL a IY (IY --> Bullet Pointer)

	ld	a,	9(iy)						;;|
	cp	#0								;;|
	ret	nz									;;| Si ya esta disparada, volvemos

	call	fire_bullet_enemy			;; Disparamos

	ret

;; ======================================================================
;; Mueve el enemigo seleccionado
;; ENTRADA: IX -> Enemigo seleccionado
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
move_enemy:

	ld	a, enemy_dir_y(ix)	;;| 
	cp	#0					;;|
	ret z					;;| If Enemy_Dir_Y == 0, it's deactivated and we return

	ld	d, enemy_y(ix)		;; D = Enemy Y Pos
	ld	a, (#enemy_vel_y)	;; A = Enemy Y Vel 
	add	a, d				;; A = Final Position (Pos + Vel)
	ld enemy_y(ix), a

	call colision_within_enemys
	cp #1
	jr nz, return_mov_y

	;; Calculamos si colisiona con el suelo
	inc		ix				;; Entity A --> Pointing at posY to do Vertical Collision
	ld		b, #197			;; B - Posicion Entity 2
	ld		c, #6			;; C - Width Entity 2
	call	colision_pared	;; Return A
	dec ix
	cp	#0					;;|
	ret	z					;;| If A != 0, Collision con Pared was made
	
	;; COLISION (Llega al final, eliminar)
	call enemy_init
	ret

	return_mov_y:
	ld a, (#enemy_vel_y)
	ld d, a
	ld a, enemy_y(ix)
	sub d
	ld enemy_y(ix), a


ret
;; ======================================================================
;; *Carga del sprite correspondiente al enemigo pasado por IX
;; ENTRADA: A
;; MODIFICA: BC, D, HL
;; SALIDA: --
;; ======================================================================
load_new_sprite:

	ld		b,	#0											;;|
	ld		c,	a											;;| Cargamos en BC el indice de la animacion

	ld 		h, enemy_anim_pointer_h(ix)						;;|
	ld 		l, enemy_anim_pointer_l(ix)						;;| Cargamos en HL un puntero al array de paletas

	add 	hl, bc											;;|
	add 	hl, bc											;;| Movemos HL a la posicion correspondiente del array 

	ld 		d, (hl)											;;|
	ld 		enemy_sprite_l(ix), d							;;| Cargamos la primera parte del sprite
															;;|
	inc 	hl												;;|
	ld 		d, (hl)											;;|
	ld 		enemy_sprite_h(ix), d							;;| Cargamos la segunda parte del sprite
	ret