
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

.include "cpctelera.h.s"
.include "keyboard.h.s"
.include "assets.h.s"
.include "macros.h.s"

inicio_pantalla:	.db #0xC0
memory_page: 		.db #0x20
pause_key:			.dw #0x0402

createpauseentity pausagui #38	#88	#24	#24	#_pre_pause

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PUBLIC FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Inicializa las variables de pintado
;; Se llama en el init del juego (main)
;; ENTRADA: --
;; MODIFICA: AF, BC, HL
;; SALIDA: --
;; ======================================================================
init_functions::
	ld 		hl, #inicio_pantalla			;; Cargamos en HL un puntero a la variable que dice en que buffer pintar
	ld 		(hl), #0xC0						;; Editamos la variable a #0xC0, la posicion que tiene inicialmente
	ld 		hl, #memory_page				;; Cargamos en HL un puntero a la variable que dice que buffer mostrar
	ld 		(hl), #0x20						;; Editamos la variable a #0x20, la variable que tiene guardada inicialmente


	ld 		a, #0x30						;;|
	ld 		l, a							;;|
	call 	cpct_setVideoMemoryPage_asm		;;| ( L / F BC HL / - )
											;;| Cambiamos la pantalla de visualizacion para solamente tener que cambiar un buffer en el main
	ret
;; ======================================================================
;; *Cambiar variables de pintado por las del siguiente buffer.
;; ENTRADA: --
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
cambiar_inicio_pantalla::

	ld 		a, (memory_page)				;;|
	xor 	#0x10							;;|
	ld 		(memory_page), a				;;|
	ld 		l, a							;;|
	call 	cpct_setVideoMemoryPage_asm		;;| ( L / F BC HL / - )
											;;| Cambiamos la variable de pantalla de visualizacion y mostramos la nueva pantalla

	call 	change_inicio_pantalla			;;| ( - / A / - )
											;;| Cambiamos la posicion en la que se van a pintar las entidades 
	ret
;; ======================================================================
;; *Cambiar la posicion en la que se van a pintar las entidades
;; ENTRADA: --
;; MODIFICA: A
;; SALIDA: --
;; ======================================================================
change_inicio_pantalla::
	ld 		a, (inicio_pantalla)			;;|
	xor 	#0x40							;;|
	ld 		(inicio_pantalla), a			;;| Cambiamos la variable de inicio_pantalla por la del siguiente buffer
	ret
;; ======================================================================
;; *Dibuja el sprite de la entidad en el buffer de pintado
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
paint_entity::

	call 	calcular_punto			;;| (IX / AF, BC, DE, HL / HL)
									;;| Calculate Screen position

	call 	solid_block_variables	;;| (IX / BC, DE, HL / BC, DE, HL)
									;;| Save Sprite Variables

	call 	cpct_drawSprite_asm		;;| (BC, DE, HL / AF, BC, DE, HL / - )
									;;| Paint the sprite
	ret
;; ======================================================================
;; *Dibuja el sprite de la entidad en los dos buffers que tiene el juego
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
paint_two_buffers_entity::
	call 	paint_entity			;;| ( IX / AF, BC, DE, HL / - )
									;;| Pintamos el sprite en el primer buffer

	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Cambiamos el buffer en el que se pintan las entidades

	call 	paint_entity			;;| ( IX / AF, BC, DE, HL / - )
									;;| Pintamos el sprite en el segundo buffer

	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Volvemos a cambiar la variable a su valor inicial
	ret
;; ======================================================================
;; *Dibuja el sprite de la entidad en el mismo buffer que se esta visualizando
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
paint_current_buffer_entity::
	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Cambiamos el buffer de pintado para que se pinte en el mismo que se muestra

	call 	paint_entity			;;| ( IX / AF, BC, DE, HL / - )
									;;| Pintamos el sprite en el mismo buffer que se muesta

	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Volvemos a cambiar la variable a su valor inicial
	ret
;; ======================================================================
;; *Pinta un bloque en ambos buffes
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
paint_two_buffers::
	call 	paint_block				;;| ( IX / AF, BC, DE, HL / - )
									;;| Pintamos un bloque en el buffer actual de pintado
	
	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Cambiamos el buffer en el que se pintan los bloques
	
	call 	paint_block				;;| ( IX / AF, BC, DE, HL / - )
									;;| Pintamos un bloque en el segundo buffer

	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Volvemos a cambiar la variable a su valor inicial
	ret
;; ======================================================================
;; *Pinta un bloque en ambos buffes
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
paint_current_buffer::	
	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Cambiamos el buffer en el que se pintan los bloque al mismo que visualizamos
	
	call 	paint_block				;;| ( IX / AF, BC, DE, HL / - )
									;;| Pintamos un bloque en el buffer que visualizamos

	call 	change_inicio_pantalla	;;| ( - / A / - )
									;;| Volvemos a cambiar la variable a su valor inicial
	ret
;; ======================================================================
;; *Dibuja un bloque con los valores correspondienes
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
paint_block::
	call 	calcular_punto			;;| (IX / AF, BC, DE, HL / HL)
									;;| Calculate Screen position
	
	call 	solid_block_variables	;;| (IX / BC, DE, HL / BC, DE, HL)
									;;| Save Sprite Variables

	ld 		a, 4(ix)				;;  Cargamos en A el color con el que se va a pintar el bloque

	call 	cpct_drawSolidBox_asm	;;| ( A, BC, DE / AF, BC, DE, HL / - )
									;;| Pintamos un bloque
	ret
;; ======================================================================
;; *Borra el sprite de la entidad
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: --
;; ======================================================================
erase_entity::
	call 	calcular_punto_borrado		;;| (IX / AF, BC, DE, HL / HL)
										;;| Calculate Screen position

	call 	solid_block_variables		;;| (IX / BC, DE, HL / BC, DE, HL)
										;;| Save Sprite Variables

	ld 		a, #0x00					;; Cargamos en A el color del fondo para borrar

	call 	cpct_drawSolidBox_asm		;;| ( A, BC, DE / AF, BC, DE, HL / - )
										;;| Pintamos un bloque
	ret
;; ======================================================================
;; * Calculamos un valor random para X
;; ENTRADA: -- 
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: A
;; ======================================================================
getRandPosX::
	call	cpct_getRandom_mxor_u8_asm		;;| ( - / AF, BC, DE, HL / L ) 
											;;| Calcula un byte random y lo guarda en L

	ld		a,	l							;;  Pasamos el byte al registro A para poder operar con ella
	and		#0x3F							;;  Aplico una mascara sobre el registro para dejar en A un valor entre 0 y 63
	add		a,	#20							;;  Le sumo el Width de la GUI. En A hay un valor entre 20 y 83
	cp 		#68								;;  Compruebo si el objeto puede salirse por la pantalla (80 - WidthBarriles = 68)
	ret 	m								;;  Si da negativo es que no puede salirse
	;; -----------------------------------------	
	;; La entidad podria salirse por el lado
	;; -----------------------------------------	
	ld 		a, l							;; Volvemos a cargar en A el byte aleatorio
	and		#0x1F							;; Aplico mascara en A un valor aleatorio de 0 a 31
	add		a,	#20							;; Le sumo el Width de la GUI. En A hay un valor entre 20 y 51, por lo que no se sale
	ret
;; ======================================================================
;; *Borra todo el contenido entre C000 - FFFF
;; ENTRADA: --
;; MODIFICA: BC, DE, HL
;; SALIDA: --
;; ======================================================================
erase_buffer_1::
	;; We copy fromn HL into DE
	ld 		hl, #0xC000	;;|
	ld 		de, #0xC001	;;| We set HL and DE
	ld 		(hl), #00	;; 	HL = 0
	ld 		bc, #0x4000	;; And we do it for all the screen (BC = Times to do)
	ldir
	ret
;; ======================================================================
;; *Borra todo el contenido entre 8000-BFFF
;; ENTRADA: --
;; MODIFICA: BC, DE, HL
;; SALIDA: --
;; ======================================================================
erase_buffer_2::
	;; We copy fromn HL into DE
	ld 		hl, #0x8000	;;|
	ld 		de, #0x8001	;;| We set HL and DE
	ld 		(hl), #00	;; HL = 0
	ld 		bc, #0x4000	;; And we do it for all the screen (BC = Times to do)
	ldir			
	ret
;; ======================================================================
;; *Borra el contenido de los dos buffers de pantalla 8000-FFFF
;; ENTRADA: --
;; MODIFICA: BC, DE, HL
;; SALIDA: --
;; ======================================================================
erase_buffers::
	call 	erase_buffer_1		;; ( - / BC, DE, HL / - )
	call 	erase_buffer_2		;; ( - / BC, DE, HL / - )
	ret
;; ======================================================================
;; *Comprobamos si el boton de pausa se esta pulsando
;; ENTRADA: -
;; MODIFICA: AF, BC, DE, HL, IX
;; SALIDA: -
;; ======================================================================
pause_detection::
	ld		hl,	(#pause_key)			;;|
	call	cpct_isKeyPressed_asm		;;|
	cp		#0x00						;;|
	jr		z,	notpaused				;;| Comprobamos si la tecla de pausa se ha pulsado
	;; -----------------------------------------
	;; Se ha pulsado la pausa
	;; -----------------------------------------
	ld		ix,	#pausagui_variables		;;|
	call	 paint_two_buffers_entity	;;| ( IX / AF, BC, DE, HL / - )
										;;| Cargamos en IX el texto de pausa y lo pintamos en los dos buffers

	call	bucle_esperar_soltar_tecla	;;| ( - / AF, BC, DE, HL / - )
										;;| Esperamos a que se suelta la tecla de pausa

	call	wait_for_enter_pause		;;| ( - / AF, BC, DE, HL / - )
										;;| Entramos en bucle hasta que se pulsa otra vez el ENTER

	ld		ix,	#pausagui_variables		;;|
	call 	paint_two_buffers			;;| ( IX / AF, BC, DE, HL / - )
										;;| Una vez salimos de la pausa borramos el mensaje de pausa
	notpaused:
	;; -----------------------------------------
	;; No se ha pulsado la pausa
	;; -----------------------------------------
	ret

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PRIVATE FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Guardamos en los registros las variables que hacen falta para pintar la entidad
;; ENTRADA: IX
;; MODIFICA: BC, DE, HL
;; SALIDA: BC, DE, HL
;; ======================================================================
solid_block_variables:
	ld 		d, h							;;| 
	ld 		e, l							;;| DE = HL | Pasamos la posicion del heroe a DL
	ld 		c, ent_width(ix)				;;  Cargamos en C el ancho del heroe
	ld 		b, ent_height(ix)			;;  Cargamos en B la altura del heroe
	ld 		h, ent_h(ix)					;;|
	ld 		l, ent_l(ix)					;;| Cargamos en HL la direccion del sprite
	ret
;; ======================================================================
;; *Calcula el punto de la pantalla correspondiente a valores X e Y del frame anterior
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: HL
;; ======================================================================
calcular_punto_borrado:
	ld 		a, (inicio_pantalla)		;;|
	ld 		d, a						;;|
	ld 		e, #0						;;| Cargamos en DE la primera posicion de la pantalla
	ld 		c, ent_last_x(ix)			;;  Cargamos en C la posicion x del frame anterior		
	ld 		b, ent_last_y(ix)			;;  Cargamos en B la posicion y del frame anterior
	call 	cpct_getScreenPtr_asm		;;| ( DE, CB / AF, BC, DE, HL / HL)
										;;| Calculo de la posicion en pantalla en funcion de X e Y
	ret
;; ======================================================================
;; *Calcula el punto de la pantalla correspondiente a valores X e Y
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL
;; SALIDA: HL
;; ======================================================================
calcular_punto:
	ld		 a, (inicio_pantalla)		;;|
	ld 		d, a						;;|
	ld 		e, #0						;;| Cargamos en DE la primera posicion de la pantalla
	ld 		c, ent_x(ix)				;;  Cargamos en C la posicion x del heroe		
	ld 		b, ent_y(ix)				;;  Cargamos en B la posicion y del heroe	
	call 	cpct_getScreenPtr_asm		;;| ( DE, CB / AF, BC, DE, HL / HL)
										;;| Calculo de la posicion en pantalla en funcion de X e Y
	ret

