
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

;; CODIGO EJEMPLO DE VARIABLE EN CODIGO

	;;modifier = .+1
	;;ld l, #0x20
	;;call cpct_setVideoMemoryPage_asm
	;;ld hl, modifier
	;;ld a, #10
	;;xor (hl)
	;;ld (modifier), a

;; ================================================================================
;; ================================================================================
;; PRIVATE DATA
;; ================================================================================
;; ================================================================================

.include "cpctelera.h.s"
.include "hero.h.s"
.include "bullet.h.s"
.include "enemy.h.s"
.include "barril.h.s"
.include "powerup.h.s"
.include "keyboard.h.s"
.include "menu.h.s"
.include "functions.h.s"
.include "keyboard/keyboard.s"
.include "niveles.h.s"
.include "piezas.h.s"
.include "assets.h.s"
.include "music.h.s"
.include "sfx.h.s"

escapekey:		.dw #0x0408
dead:			.db #0x00
win:			.db #0x00

.macro setBorder color
	;;ld hl, #0x'color'10
	;;call cpct_setPALColour_asm
.endm

;; ================================================================================
;; ================================================================================
;; PUBLIC FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================
;; Main entry point of the program
;; ======================
_main::
	
	ld sp, #0x7FFF					;; Cambia la posicion donde se almacena la pila

	ld		a,	#6					;;|
	call	change_music_speed		;;| Put the correct speed
	ld		hl,	#_song_menu			;;|
	call	change_music			;;| Put the game music

	call initialize					;; We initialize the program

	ld		hl,	#_song_battletheme	;;|
	call	change_music			;;| Put the game music

	bucle:
		setBorder 01
		call doErasing				;; Only haf at every given frame
		setBorder 02
		call save_all_last_variables

		call doUpdating				;; Only haf at every given frame

		setBorder 04
		call doDrawing				;; Only haf at every given frame
		setBorder 05

		ld		e, #0					;;| Reseteamos el valor de E para almacenar el valor del input
		ld		hl, (#escapekey)		;;| Nos guardamos el valor de la tecla d en HL
		call	cpct_isKeyPressed_asm	;;| Comprobamos si la D esta siendo pulsada
		cp		#0x00					;;|
		jp		nz, _main				;;| Si hemos pulsado ESC, salimos del juego

		ld a, (dead)
		cp #1
		jp z, escena_muerte
		ld a, (win)
		cp #1
		jp z, escena_ganar
		call cpct_waitVSYNC_asm
		
		call cambiar_inicio_pantalla
		;;call escena_ganar

		
	jr bucle

jr .

;; ======================
;; Change dead
;; ======================
change_dead::
	ld a, (dead)
	xor #1
	ld (dead), a
ret

;; ======================================================================
;; *Cambia la variable de victoria
;; ENTRADA: --
;; MODIFICA: AF
;; SALIDA: --
;; ======================================================================
change_win::
	ld 		a, (win)	;;|
	xor 	#1			;;|
	ld 		(win), a	;;| Cambiamos la variable de victoria entre 1 y 0
	ret
;; ======================
;; Escena al ganar
;; ======================
escena_ganar::
	
	ld		hl,	#_song_win	;;|
	call	change_music	;;| Poner win music

	;call change_inicio_pantalla

	call youwin_menu
	bucle_you_win_animation:
	call nave_animation
	call	cpct_scanKeyboard_asm				;; Escaneamos teclado
	ld 		hl,	#0x0402							;; Nos guardamos el valor de la tecla ENTER en HL
	call	cpct_isKeyPressed_asm				;; Comprobamos si ENTER esta siendo pulsado (A = res)
	cp		#0x00								;;|
	jr		z,	bucle_you_win_animation			;;| No se ha pulsado la tecla
	call bucle_esperar_soltar_tecla

	call change_win

	jp  _main				;; Reiniciamos el juego

;; ======================
;; We call the draw function of our entities
;; ======================
doErasing::
	call hero_erase
	call enemy_erase
	call bullet_erase_all
	call barril_erase
	call power_up_erase
	call pieza_erase
	ret

;; ======================
;; We call the draw function of our entities
;; ======================
doDrawing::
	call hero_draw
	call enemy_draw
	call bullet_draw_all
	call barril_draw
	call power_up_draw
	call pieza_draw
	call pieza_parpadeo
	ret

;; ======================
;; Guardamos todas las variables para el segundo buffer
;; ======================
save_all_last_variables::
	call save_last_variables_bullet
	call save_last_variables_hero
	call save_last_variables_enemy
	call save_last_variables_barril
	call save_last_variables_power_up
	call save_last_variables_pieza
ret

;; ================================================================================
;; ================================================================================
;; PRIVATE FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================
;; We initialize the game
;; ======================
initialize:

	call cpct_disableFirmware_asm
	ld c, #0x00
	call cpct_setVideoMode_asm
	ld hl, #_pre_palette
	ld de, #16
	call cpct_setPalette_asm

	
	ld hl, #0x1610							;;|
	call cpct_setPALColour_asm				;;| Set Border Colors

	
	call init_music							;; We init the music

	
	ld		hl,	#isr						;;|
	call	cpct_setInterruptHandler_asm	;;| Set ISR

	call init_functions						;;| ( - / AF, BC, HL / - )
											;;| Inicializa las funciones de pintado

	call erase_buffer_1

	;; Remap Keys
	init_game_options:
	call 	menu_draw_title
	call 	wait_input_menu					;; Metodo en el que se elige el metodo de juego
	cp 		#0x00							;; A==0 Cargamos las teclas
	jr 		nz, no_cargar_teclas

	ld 		ix, #hero_input
	call 	cargar_teclas

	ld hl, #0x1610							;;|
	call cpct_setPALColour_asm				;;| Set Palette

	call 	erase_buffer_1

	jr 		init_game_options

	no_cargar_teclas:						;; A puede ser 1 o 2
	call change_mode_hero_update

	
	ld hl, #0x1410							;;|
	call cpct_setPALColour_asm				;;| Set Border Colors

	call erase_buffers						;; Clean the buffers
	call init_variables_game				;; Initialize Entity
	
ret

;; ======================
;; Init variables
;; ======================
init_variables_game:

	call init_gui	
	call gui_draw
	call niveles_init
	call bullet_init_all
	call barril_init_all
	call enemy_desactive
	call enemy_init_movement
	call enemy_init_all
	call hero_init
	call power_up_init
	call pieza_init_partida

	ret

;; ======================
;; Change hero update  Keyboard/Joystick
;; ======================
change_mode_hero_update:

	;; Codigo para cambiar la variable del update hero
	cp #2							;; El numero viene del pasado por keyboard.s -> wait_for_enter
	jr z, change_to_keyboard		;; Si A==2 pasamos a keyboard / Si A!=2 pasamos a joystick


	call change_keys_hero
	ld de, #hero_update_joystick
	jr change_mode

	change_to_keyboard:
	ld de, #hero_update

	change_mode:
	ld hl, #metodo_update
	ld (hl), e
	inc hl
	ld	(hl),	d
	
	ret

;; ======================
;; We call the update function of our entities
;; ======================
doUpdating:
	call pieza_aparicion
	call bullet_update_all
	call pup_update_animation

	call cpct_waitVSYNC_asm

	metodo_update = .+1		;; Variable en el propio codigo
	call #hero_update
	call enemy_update

ret

;; ======================
;; ISR para musica y scans
;; ======================
isr:
	call cpct_scanKeyboard_if_asm
	call isr_music
ret

;; ======================
;; Escena de muerte
;; ======================
escena_muerte:
	call change_dead
	call gameover_menu
	call wait_for_enter	
	jp _main