
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "cpctelera.h.s"
.include "assets.h.s"
.include "sfx.h.s"

current_music:	.dw #_song_menu

mute:			.db #0x00
mute_key:		.dw #0x4004
mute_counter:	.db #0x15

music_counter:	.db #0x6
music_rate::	.db #0x6


;; ================================================================================
;; ================================================================================
;; PUBLIC FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================
;; Iniciamos la musica
;; ======================
init_music::
	ld	hl,	#current_music
	ld 	e, (hl)
	inc hl
	ld	d, (hl)

	call cpct_akp_musicInit_asm
	ld	de,	#_song_sfx
	call init_sfx
ret

;; ======================
;; Cambiamos la cancion
;; ENTRADA: hl --> New song
;; ======================
change_music::
	push hl
	pop  de

	ld	hl,	#current_music
	ld (hl), e
	inc hl
	ld (hl), d
	
	ld	hl,	#mute		;;|
	ld	a,	(hl)		;;|
	cp	#0				;;|
	call z, init_music	;;| Always change, only init if not mute
ret

;; ======================================================================
;; *Cambia la velocidad de la musica
;; ENTRADA: A
;; MODIFICA: HL
;; SALIDA: --
;; ======================================================================
change_music_speed::
	ld		hl,	#music_rate		;;|
	ld		(hl), a				;;| Cambiamos la velocidad de la musica
ret

;; ======================
;; Reproducimos la musica
;; ======================
isr_music::

	ex	af,	af'
	exx
	push	af
	push	bc
	push	de
	push	hl
	push	iy

	ld	a,	(music_counter)					;;|
	dec	a									;;|
	ld	(music_counter),	a				;;| Restamos 1 al contador, (si no es 0 volvemos)
	jp nz, return_music

	ld	hl,	#mute
	ld	a,	(hl)
	cp	#1
	jr	nz,	dontreset
		ld	de,	#_song_sfx
		call cpct_akp_musicInit_asm

	dontreset:
	call	cpct_akp_musicPlay_asm			;; Ponemos la musica
	;;call	stop_sfx


	ld	hl, #music_rate
	ld	a,	(hl)							;;|
	ld	(music_counter),	a				;;| Y ponemos el caontador a 6	

	call mute_detection						;; Vemos si tenemos que mutear

	return_music:
	pop iy
	pop hl
	pop de
	pop bc
	pop af
	exx
	ex	af, af'

ret

;; ======================
;; MUTE detection
;; ======================
mute_detection:

	ld		hl, 	#mute_counter	;;|
	ld		a,		(hl)			;;|
	cp		#0						;;|
		jr	z,		checkmute		;;|
	dec		a						;;|
	ld		(hl),	a				;;| Counter to check Mute key
	jr		donotchange

	checkmute:
	ld		e, #0					;; Reseteamos el valor de E para almacenar el valor del input
	ld		hl, (#mute_key)			;; Nos guardamos el valor de la tecla d en HL
	call	cpct_isKeyPressed_asm	;; Comprobamos si esta pulsada
	cp		#0x00
	jr z, donotchange

		ld	hl,	#mute
		ld	a,	(hl)
		cp #0
		jr z, domute
			call init_music			;; Seleccionamos la nueva cancion
			ld a, #0
			jr endchanging
		domute:
		ld	de,	#_song_sfx
		call cpct_akp_musicInit_asm
		ld a, #1

		endchanging:
		ld		hl, #mute
		ld		(hl),	a

		ld		a,		#15				;;|
		ld		hl, 	#mute_counter	;;|
		ld		(hl),	a				;;| Reset counter

	donotchange:

ret

;; ================================================================================
;; ================================================================================
;; PRIVATE FUNCTIONS
;; ================================================================================
;; ================================================================================