
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

	.include "enemy.h.s"
	.include "hero.h.s"
	.include "menu.h.s"
	.include "piezas.h.s"
	.include "assets.h.s"
	.include "cpctelera.h.s"
	.globl change_win

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PRIVATE DATA
;; =====================================================================================================================||
;; =====================================================================================================================||

	nivel_actual: 			.db #1
	enemigos_pasar_nivel: 	.db #5
	enemigos_max_matar:		.db #5

	lista_paletas:
		paleta_1: .dw #_pre_palette_1
		paleta_2: .dw #_pre_palette_2
		paleta_3: .dw #_pre_palette_3
		paleta_4: .dw #_pre_palette_4
		paleta_5: .dw #_pre_palette_5
		paleta_6: .dw #_pre_palette_6

	index_paleta: .db #0

	paleta_anterior:: 	.dw #_pre_palette
	paleta_actual::		.dw #_pre_palette

;; =====================================================================================================================||
;; =====================================================================================================================||
;; PUBLIC FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Inicializa las variables de nivel
;; ENTRADA: --
;; MODIFICA: AF, DE, HL
;; SALIDA: --
;; ======================================================================
niveles_init::
	ld 		hl, #nivel_actual				;;|	
	ld 		(hl), #1						;;| Guardamos en nivel actual el 1

	ld 		a, #5							;;|
	ld 		(enemigos_pasar_nivel), a		;;|
	ld 		(enemigos_max_matar), a			;;| Ponemos los valores por defecto de enemigos por matar en cada nivel

	ld 		a, #0							;;|
	ld 		(index_paleta), a				;;| Actualizamos el indice de paleta a 0

	ld 		hl, #_pre_palette				;;|
	ld 		d, h							;;|
	ld 		e, l							;;| Nos guardamos la paleta a utilizar en DE
	ld 		hl, #paleta_actual				;;|
	ld 		(hl), e							;;|
	inc 	hl								;;|
	ld 		(hl), d							;;| Almacenamos esta paleta en la variable correspondiente

	ret
;; ======================================================================
;; *Decrementa el numero de enemigos por matar este nivel
;; ENTRADA: --
;; MODIFICA: A, IX
;; SALIDA: --
;; ======================================================================
dec_enemy_nivel::
	ld 		a, (enemigos_pasar_nivel)	;;| 
	dec 	a							;;|
	ld 		(enemigos_pasar_nivel), a	;;| Decrementamos el valor de enemigos por matar
	ret 	nz							;;| Comprobamos si el numero de enemigos llega a 0

	call 	pieza_change_aparition		;;| ( - / IX / - )
										;;| En el caso de que llegue a 0 hacemos que pueda aparecer la pieza
	ret
;; ======================================================================
;; *Pasamos al siguiente nivel
;; ENTRADA: --
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
change_level::
	call 	cambiar_paleta					;;| ( - / AF, BC, HL / - )
											;;| Cambia la paleta de parpadeo al subir de nivel

	ld 		a, (enemigos_max_matar)			;;| 
	add 	a, #5							;;| 
	ld 		(enemigos_max_matar), a			;;| Poner el nuevo valor de enemigos por matar
	ld 		(enemigos_pasar_nivel), a		;;| 5 enemigos mas que en nivel anterior

	ld 		a, (nivel_actual)				;;| 
	inc 	a								;;| Incrementamos el nivel
	ld 		(nivel_actual), a				;;| Guardamos el nuevo valor del nivel
	cp 		#10								;;|
	jr 		z, acabar_juego					;;| En el caso de llegar al nivel maximo acabamos el juego
	
	push 	af								;;| Almacenamos AF en la pila para no modificarlo en el call
	call 	draw_ship						;;| ( - / AF, BC, DE, HL / - )
	pop 	af								;;| Recogemos el AF de la pila

	cp 		#3								;;|
	ld 		ix, #enemy4_data				;;| Si es el nivel 3 activamos el enemigo 4
	jr 		z, aparacicion_enemigo			;;| ( IX / AF, BC, DE, HL, IY / - )
	
	cp 		#5								;;|
	ld 		ix, #enemy5_data				;;| Si es el nivel 5 activamos el enemigo 5
	jr 		z, aparacicion_enemigo			;;| ( IX / AF, BC, DE, HL, IY / - )

											;;| En los otros casos de los niveles
	call 	enemy_buff						;;| ( - / AF / - )
											;;| Incrementar variables de los enemigos

	change_level_commands:
	;; -----------------------------------------
	;; Continuar cambiado nivel
	;; -----------------------------------------	
	ld 		a, (nivel_actual)				;;  Guardamos en A el nivel actual
	call 	gui_lvl_up						;;| ( A / AF, BC, DE, HL / - )
											;;| Actualizamos el numero del nivel
	call 	add_life						;;| ( - / AF, BC, DE, HL / - )
											;;| Le anyadimos una vida al heroe
	ret
	acabar_juego:
	;; -----------------------------------------
	;; Acaba el juego
	;; -----------------------------------------	
	call 	change_win						;;|  ( - / AF / - )
											;;| Alteramos la variable de victoria
	ret
;; =====================================================================================================================||
;; =====================================================================================================================||
;; PRIVATE FUNCTIONS
;; =====================================================================================================================||
;; =====================================================================================================================||

;; ======================================================================
;; *Activamos la aparicion de uno de los enemigos
;; ENTRADA: IX
;; MODIFICA: AF, BC, DE, HL, IY
;; SALIDA: --
;; ======================================================================
aparacicion_enemigo:
	ld 		10(ix), #1				;;| Cambiamos la variable de aparicion del enemigo
	call 	enemy_init				;;| ( IX / AF, BC, DE, HL, IY / - )
	jr 		change_level_commands	;;| Continuamos con los cambios de change level
;; ======================================================================
;; *Cambiamos la paleta con la que parpadeara al subir de nivel
;; ENTRADA: --
;; MODIFICA: AF, BC, HL
;; SALIDA: --
;; ======================================================================
cambiar_paleta:

	ld 		hl, #lista_paletas		;;  Cargamos en HL un puntero al array de paletas
	ld 		a, (index_paleta)		;;|
	ld 		c, a					;;|	
	ld 		b, #00					;;| Cargamos en BC el indice de la paleta

	add 	hl, bc					;;|
	add 	hl, bc					;;| Movemos HL a la posicion correspondiente del array 

	ld 		a, (hl)					;;|
	ld 		b, a					;;|
	inc 	hl						;;|
	ld 		a, (hl)					;;| Cargamos en AB la paleta a utilizar

	ld 		hl, #paleta_actual		;;|
	ld 		(hl), b					;;|
	inc 	hl						;;|
	ld 		(hl), a					;;| Cargamos en paleta_actual la nueva paleta a utilizar

	ld 		a, (index_paleta)		;;|
	inc 	a						;;|
	ld 		(index_paleta), a		;;| Incrementamos el indice de la paleta
	cp 		#6						;;|
	ret 	nz						;;| Comprobamos si el indice ha llegado al maximo

	ld 		hl, #index_paleta		;;|
	ld 		(hl), #0				;;| En el caso de que llegue al maximo volvemos al valor inicial
	ret

