
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

.include "cpctelera.h.s"
.include "collision.h.s"
.include "functions.h.s"
.include "assets.h.s"
.include "menu.h.s"
.include "sfx.h.s"
.include "niveles.h.s"
.include "macros.h.s"

;; ================================================================================
;; ================================================================================
;; PRIVATE DATA
;; ================================================================================
;; ================================================================================

crearPieza pieza1 76, 0, 4, 12, #_pre_wrench, #0

;; ================================================================================
;; ================================================================================
;; PUBLIC FUCTIONS
;; ================================================================================
;; ================================================================================

;; ======================================================================
;; Pinta la pieza
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pieza_init_partida::
	ld ix, #pieza1_variables
	ld pieza_enabled(ix), #0
	ld pieza_aparition(ix), #0
	ld pieza_x(ix), #0
	ld pieza_y(ix), #170

	ld a, #0
	ld (contador_parpadeo), a

	ld a, #1
	ld (contador_parpadeo_2), a

	ld a, #1
	ld (seleccionador_paleta), a

	ret

;; ======================================================================
;; Pinta la pieza
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pieza_aparicion::
	ld ix, #pieza1_variables
	ld a, pieza_enabled(ix)
	cp #0
	ret nz
	ld a, pieza_aparition(ix)
	cp #0
	ret z

	call	cpct_getRandom_mxor_u8_asm		;; We fetch a rnd [0, 7]
	ld		a,	l							;; A = L = Rnd Num
	ld		b,	#0x3F						;; Mascara para coger 2 bits
	;ld		b,	#0x00						;; Mascara para coger 2 bits
	and		b								;; A = [0, 7]

	cp	#0									;; Check against 0, (12'5% of spawning)

	call z, pieza_init


	ret
;; ======================================================================
;; *Cambia la variable de aparacion de la pieza
;; ENTRADA: --
;; MODIFICA: IX
;; SALIDA: --
;; ======================================================================
pieza_change_aparition::
	ld 		ix, #pieza1_variables		;;|
	ld 		pieza_aparition(ix), #1		;;| Cambia la variable de aparacion de la pieza a 1
	ret
;; ======================================================================
;; Pinta la pieza
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pieza_init::

	ld	ix,	#pieza1_variables

	ld	pieza_y(ix),			#0

	random_x_value:
	call	getRandPosX
	ld		pieza_x(ix),		a			;; Seguidamente guardamos la X y comprobamos si colisiona con algo
	call	colision_within_entities
	cp		#0
	jr		z,	random_x_value				;; En el caso en que colisione volvemos a calcular la x

	ld	pieza_enabled(ix),		#2

	call save_last_variables_pieza

	ret

;; ======================================================================
;; Pinta la pieza
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================

pieza_draw::
	ld	 ix,	#pieza1_variables
	ld	  a,	pieza_enabled(ix)		;; We get the activation flag
	cp	  a,	#2
	ret nz
	call paint_entity

	ret

;; ======================================================================
;; Guarda las variables de la pieza
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================

save_last_variables_pieza::
	ld	ix,	#pieza1_variables
	
	ld	a, pieza_x(ix)
	ld	pieza_last_x(ix), a

	ld	a, pieza_y(ix)
	ld	pieza_last_y(ix), a
ret

;; ======================================================================
;; Parpadeo colores
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================

pieza_parpadeo::
	contador_parpadeo = .+1
	ld a, #0x00
	cp #0
	ret z							;; Comprobamos si aun le quedan iteraciones por hacer al contador

	dec a
	ld (contador_parpadeo), a		;; Decrementamos el contador
	push af
	call paint_text_lvl_up
 	pop af
	cp #0
	call z, erase_text_lvl_up

	contador_parpadeo_2 = .+1
	ld a, #1
	dec a
	ld (contador_parpadeo_2), a
	ret nz							;; Segundo contador, cada 16 cambio de paleta

	ld a, #16
	ld (contador_parpadeo_2), a

	seleccionador_paleta = .+1
	ld a, #1
	ld c, a
	ld b, #00
	ld hl, #paleta_anterior			;; Cargamos las variables

	add hl, bc
	add hl, bc

	ld c, (hl)
	inc hl
	ld b, (hl)

	ld h, b
	ld l, c

	ld de, #16
	call cpct_setPalette_asm

	ld a, (seleccionador_paleta)
	xor #1
	ld (seleccionador_paleta), a

	ret


;; ======================================================================
;; Borra la pieza
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================

pieza_erase::

	ld		ix,	#pieza1_variables
	ld		a,	pieza_enabled(ix)	;;| 
	cp		#0						;;|
	ret 	z

	cp		#1
	jr 		nz, go_to_erase
	ld 		pieza_enabled(ix), #0

	go_to_erase:
	call	erase_entity		;;| If Barrel type != 0, it's activated & we erase the barrel

	ret

;; ======================================================================
;; Update del pup
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pieza_update::

	ld	ix,	#pieza1_variables			;; Cargamos el power_up

	ld	 a,	pieza_enabled(ix)			;;| 
	cp	#0								;;|
	ret z								;;| If Enemy type = 0, it's deactivated and we return

	ld	a, pieza_y(ix)					;; A = PUP Y Pos
	add	a, #2							;; Le sumamos la velocidad a la Y Pos

	ld d, a								;; D = A, Nos guardamos la posicion actualizada en D
	push de

	;; Calculamos si colisiona con el suelo
	inc		ix							;; Entity A --> Pointing at posY to do Vertical Collision
	ld		b, #197						;; B - Posicion Entity 2
	ld		c, #6						;; C - Width Entity 2
	call	colision_pared				;; Return A

	cp	#0								;;|
	pop	de								;;|
	jr	z, sin_colision					;;| If A != 0, Collision con Pared was made
	
	;; COLISION
	dec ix
	ld pieza_enabled(ix), #1			;; Desactivar pieza

	ret
	
	;; NO COLISION
	sin_colision:		
	ld a, d						;; Sacamos la posicion actualziada de D a A (A = D)
	ld 0(ix), a					;; La guardamos como nueva posicion Y

	ret

;; ======================================================================
;; Pieza pillada
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================

pieza_pillada::

	push	ix
	ld		ix,	#pieza1_variables
	ld		a,	pieza_enabled(ix)
	pop		ix
	cp 		#0
	ret		z

	call play_lvl_sfx
	call pieza_deactivate
	call change_level
	ld a, #64
	ld (contador_parpadeo), a
	ret

;; ================================================================================
;; ================================================================================
;; PRIVATE FUCTIONS
;; ================================================================================
;; ================================================================================

;; ======================================================================
;; Desactivar ficha
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pieza_deactivate:
	ld pieza_enabled(iy), #1
	ld pieza_aparition(iy), #0
	ld pieza_x(iy), #0
	ld pieza_y(iy), #170
	ret

;; ======================================================================
;; Pieza pillada
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pieza_desactivar::
	ld ix, #pieza1_variables
	ld a, pieza_aparition(ix)
	cp #0
	jr z, charge_others_variables
	ld a, pieza_enabled(ix)
	cp #0
	jr z, charge_others_variables

	ld pieza_enabled(ix), #1			;; Desactivar piezald pieza_enabled(iy), #1
	

	charge_others_variables:
	ld pieza_x(ix), #0
	ld pieza_y(ix), #170
ret