
;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of ProtonSmoke: An Amstrad CPC Shoot 'Em Up Game
;; Copyright (C) 2017 Daniel Finestrat Martinez (@DanielFinestrat)
;; Copyright (C) 2017 Angel Navarro Martinez (@EingelNavarro)
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _CODE

.include "cpctelera.h.s"
.include "collision.h.s"
.include "functions.h.s"
.include "assets.h.s"

.include "macros.h.s"

;; ================================================================================
;; ================================================================================
;; PRIVATE DATA
;; ================================================================================
;; ================================================================================

;; Pup animation
pup_animation:
	pup_0: 		.dw	#_pre_powerUp1
	pup_1:		.dw #_pre_powerUp1
	pup_2:		.dw #_pre_powerUp1
	pup_3: 		.dw	#_pre_powerUp2
	pup_4:		.dw #_pre_powerUp2
	pup_5:		.dw #_pre_powerUp2

;; pup Definition
crearPowerUp power_up, #70, #0, #4, #10, #_pre_powerUp1, #0

;; ================================================================================
;; ================================================================================
;; PUBLIC FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================================================================
;; Inicializa el pup
;; MODIFICA: IX
;; ======================================================================
power_up_init::
	ld	ix,	#power_up_variables
	ld	pup_x(ix),			#70
	ld	pup_y(ix),			#0
	ld	pup_enabled(ix),	#1
	ld	pup_anim_index(ix),	#0
ret

;; ======================================================================
;; Spawnea el pup
;; ENTRADA: IX --> Enemigo
;; MODIFICA: IY
;; ======================================================================
power_up_spawn::

	ld	iy,	#power_up_variables		;; Cargamos el power Up
	ld	a,	pup_enabled(iy)			;;|
	cp	#0							;;|
	ret	nz								;;| Si esta activado (A != 0), volvemos

	ld	a,	0(ix)					;;|
	ld	pup_x(iy),			a		;;| Cargamos la X

	ld	a,	1(ix)					;;|
	ld	pup_y(iy),			a		;;| Cargamos la y

	ld	pup_enabled(iy),	#2		;; Lo activamos
ret

;; ======================================================================
;; Update del pup
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
power_up_update::

	ld	ix,	#power_up_variables			;; Cargamos el power_up

	ld	 a,	pup_enabled(ix)		;;| 
	cp	#0						;;|
	ret z						;;| If Enemy type = 0, it's deactivated and we return

	ld	a, pup_y(ix)			;; A = PUP Y Pos
	add	a, #2					;; Le sumamos la velocidad a la Y Pos

	ld d, a						;; D = A, Nos guardamos la posicion actualizada en D
	push de

	;; Calculamos si colisiona con el suelo
	inc		ix					;; Entity A --> Pointing at posY to do Vertical Collision
	ld		b, #197				;; B - Posicion Entity 2
	ld		c, #6				;; C - Width Entity 2
	call	colision_pared		;; Return A

	cp	#0						;;|
	pop	de						;;|
	jr	z, sin_colision			;;| If A != 0, Collision con Pared was made
	
	;; COLISION
	dec ix
	call power_up_init

ret
	
	;; NO COLISION
	sin_colision:		
	ld a, d						;; Sacamos la posicion actualziada de D a A (A = D)
	ld 0(ix), a					;; La guardamos como nueva posicion Y

ret

;; ======================================================================
;; Guarda las variables del frame anterior
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
save_last_variables_power_up::
	ld	ix,	#power_up_variables
	
	ld	a, pup_x(ix)
	ld	pup_last_x(ix), a

	ld	a, pup_y(ix)
	ld	pup_last_y(ix), a
ret

;; ======================================================================
;; Borra el pup
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
power_up_erase::
	ld		ix,	#power_up_variables
	ld		 a,	pup_enabled(ix)		;;| 
	cp		#0						;;|
	ret z
	cp		#1
	jr nz, go_to_erase
	ld 		pup_enabled(ix), #0

	go_to_erase:
	call	erase_entity		;;| If Barrel type != 0, it's activated & we erase the barrel
ret

;; ======================================================================
;; Pinta el pup
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
power_up_draw::
	ld	 ix,	#power_up_variables
	ld	  a,	pup_enabled(ix)		;; We get the activation flag
	cp	  a,	#2
	ret nz
	call paint_entity
ret

;; ======================================================================
;; Update del sprite del heroe
;; MODIFICA: AF, BC, DE, HL
;; ======================================================================
pup_update_animation::

	ld	ix,	#power_up_variables			;; Cargamos el power_up

	ld	 a,	pup_enabled(ix)				;;| 
	cp	#0								;;|
	ret z								;;| If 0, it's deactivated and we return


	ld 		a, pup_anim_index(ix)
	inc 	a
	ld 		pup_anim_index(ix), a
	cp 		#6
	jr		nz, cargar_sprite_nuevo
	
		ld		a, #0
		ld 		pup_anim_index(ix), a

	cargar_sprite_nuevo:
	call	load_new_sprite

ret

;; ================================================================================
;; ================================================================================
;; PRIVATE FUNCTIONS
;; ================================================================================
;; ================================================================================

;; ======================================================================
;; Carga en IX, el sprite correspondiente al numero pasado por A*2
;; ENTRADA: A, IX
;; MODIFICA: IX, A --> Que sprite seleccionamos
;; ======================================================================
load_new_sprite:

	ld	b,	#0											;;|
	ld	c,	a											;;| BC --> A

	ld h, pup_anim_pointer_h(ix)						;;|
	ld l, pup_anim_pointer_l(ix)						;;| Pointer to current animation frame

	add hl, bc											;;|
	add hl, bc											;;| We select the next frame (2 bytes after)

	ld d, (hl)											;;|
	ld pup_sprite_l(ix), d								;;| We load the sprite Low byte

	inc hl												;;|
	ld d, (hl)											;;|
	ld pup_sprite_h(ix), d								;;| We load the sprite High byte

ret